# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['teselagen',
 'teselagen.api',
 'teselagen.api.tests',
 'teselagen.examples',
 'teselagen.utils']

package_data = \
{'': ['*'],
 'teselagen': ['.mypy_cache/*',
               '.mypy_cache/3.6/*',
               '.mypy_cache/3.6/_typeshed/*',
               '.mypy_cache/3.6/collections/*',
               '.mypy_cache/3.6/email/*',
               '.mypy_cache/3.6/http/*',
               '.mypy_cache/3.6/importlib/*',
               '.mypy_cache/3.6/json/*',
               '.mypy_cache/3.6/logging/*',
               '.mypy_cache/3.6/os/*',
               '.mypy_cache/3.6/requests/*',
               '.mypy_cache/3.6/requests/packages/*',
               '.mypy_cache/3.6/requests/packages/urllib3/*',
               '.mypy_cache/3.6/requests/packages/urllib3/packages/*',
               '.mypy_cache/3.6/requests/packages/urllib3/packages/ssl_match_hostname/*',
               '.mypy_cache/3.6/requests/packages/urllib3/util/*',
               '.mypy_cache/3.6/teselagen/*',
               '.mypy_cache/3.6/teselagen/api/*',
               '.mypy_cache/3.6/teselagen/utils/*',
               '.mypy_cache/3.6/urllib/*'],
 'teselagen.examples': ['.ipynb_checkpoints/*',
                        '_dev/*',
                        '_dev/.ipynb_checkpoints/*',
                        'pytested/*',
                        'pytested/.ipynb_checkpoints/*']}

install_requires = \
['dna_features_viewer>=3.0.3,<4.0.0',
 'fastaparser>=1.1,<2.0',
 'pandas>=1.0.0,<2.0.0',
 'pytest-cov>=2.8.1,<3.0.0',
 'pytest-notebook>=0.6.1,<0.7.0',
 'pytest-xdist>=1.31.0,<2.0.0',
 'pytest>=5.3.5,<6.0.0',
 'requests-mock>=1.8,<2.0',
 'requests>=2.22.0,<3.0.0',
 'setuptools>=45.1.0,<46.0.0',
 'single_version>=1.5.1,<2.0.0',
 'tqdm>=4.53.0,<5.0.0']

setup_kwargs = {
    'name': 'teselagen',
    'version': '0.3.2',
    'description': 'Teselagen Biotechnology API client',
    'long_description': '# TeselaGen Python Tools.\n\nThis package includes some Python tools to use in combination with TeselaGen platform.\nThese tools includes _TeselaGen Python API Client_ \n\nThis package runs on Python 3.6 and above.\n\n**NOTE :** All the following commands are supposed to be run on the `base` directory, unless specified.\n\n## Library Installation\nThis library contains the TeselaGen Python API Client.\n\nTo install it locally,\n\n1. By using PIP\n\n    Use `pip install teselagen`\n\n\n',
    'author': 'Your Name',
    'author_email': 'you@example.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/TeselaGen/api-client',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



#!/usr/local/bin/python3
# Copyright (C) 2018 TeselaGen Biotechnology, Inc.
# License: MIT
import getpass
import json
from pathlib import Path
import time
from typing import Any, Dict, List, Optional, Tuple, Union

import requests
from teselagen.api.design_client import DESIGNClient
from teselagen.api.discover_client import DISCOVERClient
from teselagen.api.test_client import TESTClient
from teselagen.utils import DEFAULT_API_TOKEN_NAME
from teselagen.utils import DEFAULT_HOST_URL
from teselagen.utils import get
from teselagen.utils import get_credentials
from teselagen.utils import get_credentials_path
from teselagen.utils import load_from_json
from teselagen.utils import post
from teselagen.utils import put
from teselagen.utils import requires_login

AVAILABLE_MODULES: List[str] = ["test", "evolve"]  # ["test", "learn"/"evolve"]

# DEFAULT_HOST_URL: str = "https://platform.teselagen.com"
# DEFAULT_API_TOKEN_NAME: str = "x-tg-cli-token"

# NOTE : Related to Postman and Python requests
#       "body" goes into the "json" argument
#       "Query Params" goes into "params" argument


# TODO: Maybe is better to set a default value for expires_in = "30m" instead of "1d" (?) or 8 hours
class TeselaGenClient():
    """Python TeselaGen Client."""

    def __init__(
        self,
        host_url: str = DEFAULT_HOST_URL,
        api_token_name: str = DEFAULT_API_TOKEN_NAME,
        module_name:
        str = "design"  # NOTE: For cross-module endpoints use the DESIGN module as default.
    ) -> None:
        """

        A Python Client to use for communication with the TeselaGen modules.

        Args :
            module (str) : The module name to use for communication.

                    Available Modules :
                                        "test"
                                        "discover",
                                        "design",
                                        "build" (WIP)

            host_url (str) : The Host URL of the API.

                Default = "https://platform.teselagen.com"

            api_token_name (str) : The name of the API token to use.

                Default = "x-tg-cli-token"

        """

        self._design = None
        self._test = None
        self._discover = None
        # NOTE: Do not add passwords to the class attributes.
        #       Delete all passwords once they've been used.

        self.host_url: str = host_url
        self.api_token_name: str = api_token_name

        # Here we define a common Base URL. Using the DESIGN Module as the target server for these common endpoints.
        _module_name: str = module_name if module_name is not "discover" else "evolve"
        module_url: str = f"{self.host_url}/{_module_name}"
        api_url_base: str = f"{module_url}/cli-api"

        # Here we define the client endpoints. Using the DESIGN Module as the target server for these common endpoints.
        self.register_url: str = f"{module_url}/register"
        self.login_url: str = f"{module_url}/login"
        self.info_url: str = f"{api_url_base}/info"
        self.status_url: str = f"{api_url_base}/public/status"
        self.auth_url: str = f"{api_url_base}/public/auth"

        # Laboratories
        self.labs_url: str = f"{self.host_url}/test/cli-api/laboratories"

        # NOTE : The authorization token will be updated with the
        #        "login" method.
        self.auth_token: Optional[str] = None

        # Here we define the headers.
        self.headers: Dict[str, str] = {"Content-Type": "application/json"}

    # The next four properties are TG Module Classes providing a series of functions that interact with their corresponding TG API endpoints.
    # These objects are instantiated with the TeselaGen Client object so they share all common functions (s.a. login, logout, register, select/unselect lab).

    @property
    def design(self):
        """
        This instantiates the client's 'design' property object which provides TeselaGen DESIGN API methods.
        """
        if self._design is None:
            self._design = DESIGNClient(teselagen_client=self)
        return self._design

    @property
    def build(self):
        """
        This instantiates the client's 'build' property object which provides TeselaGen BUILD API methods.
        """
        raise NotImplementedError("Build module client is not yet implemented.")

    @property
    def discover(self):
        """
        This instantiates the client's 'discover' property object which provides TeselaGen DISCOVER API methods.
        """
        if self._discover is None:
            self._discover = DISCOVERClient(teselagen_client=self)
        return self._discover

    @property
    def test(self):
        """
        This instantiates the client's 'test' property object which provides TeselaGen TEST API methods.
        """
        if self._test is None:
            self._test = TESTClient(teselagen_client=self)
        return self._test

    # Common methods for all four TG Modules.

    def register(self, username: str, password: str):
        """
        Registers a new user.

        NB: Registering a new user might require ADMIN priviledges.
        """
        body = {
            "email": username,
            "firstName": "test",
            "lastName": "user",
            "password": password,
            "passwordConfirm": password
        }
        response: Dict[str, Any] = post(url=self.register_url, json=body)
        response["content"] = json.loads(response["content"])
        return response

    def login(
        self,
        username: Optional[str] = None,
        password: Optional[str] = None,
        apiKey: Optional[str] = None,
        expiration_time: str = "1d",
    ) -> None:
        """
            Login to the CLI with the username used to login through the UI.
            A password or an apiKey is required. If none is provided password will be prompted.

            Args:
                username (Optional[str]) : A valid username (usually their email)
                    to authenticate. If not provided, it will be prompted.

                    Default : None

                password (Optional[str]) : A password for the user. If not provided
                    it will be prompted.

                    Default: None

                apiKey (Optional[str]) : An exclusive API password obtained from the TeselaGen Browser Application Settings.
                    It has 1 day expiration.

                    Default: None

                expiration_time (Optional[str]) : Expiration time for the
                    authentication (token), in zeit/ms format.

                    Default = "1d"
        """
        # NOTE: the apiKey is obtained as an alternative password with 1 day expiration.
        _password = apiKey if apiKey is not None else password
        username, password = get_credentials(
            username=username,
            password=_password,
        )
        auth_token = self.create_token(
            username=username,
            password=password,
            expiration_time=expiration_time,
        )
        del username, password
        # else:
        #     auth_token = apiKey
        # It will update the auth token and headers.
        self.update_token(token=auth_token)
        return None

    def logout(self,
               username: Optional[str] = None,
               password: Optional[str] = None) -> None:
        """
        Log out from the CLI. A password is required for comfirmation.

        Args:

            username (Optional[str]) : Username. If not provided, it will be
                prompted.

            password (Optional[str]) : Password. If not provided, it will be
                prompted.

        """
        # TODO : Implement a new endpoint to deauthorize a token.

        # We locally delete the last token.
        self.update_token(token=None)

        username, password = get_credentials(username=username,
                                             password=password)

        # We create a temporary token, and wait until it expires.
        _ = self.create_token(username=username,
                              password=password,
                              expiration_time="1s")

        del username, password

        # We wait (a few seconds) for the (temporary) token to expire
        time.sleep(3)

        # NOTE :Verify that the user is deauthorized after the return.
        # raise NotImplementedError
        return

    def get_server_status(self) -> str:
        """

        Gets the current Server Status.

        Returns:

        """
        response: Dict[str, Any] = get(
            url=self.status_url,
            headers=self.headers,
        )

        return response["content"]

    def create_token(
        self,
        username: str,
        password: str,
        expiration_time: str,
    ) -> Union[str, None]:
        """

            Create a new access token for the user with the given username,
            password and expiration time, and return the new access token.

            Args:

                username (str) : The username identifier to authenticate with the
                    API.

                password (str) : The password identifier to authenticate with the
                    API.


                expiration_time (str) : Expiration time for the authentication
                    (token), in zeit/ms format.

                    Example : "1d"

            Returns:
                (Union[str, None]) : It returns the authentication token (as a
                    string) for the given email address, or None if the email
                    address is not authenticated.
        """
        body = {
            "username": username,
            "password": password,
            "expiresIn": expiration_time
        }

        # This happens in the CLI
        try:
            response: Dict[str, Any] = put(
                url=self.auth_url,
                headers=self.headers,
                json=body,
            )
        except Exception as e:
            # TODO : Use a logger
            print("Connection Refused")
            return None
        print("Connection Accepted")

        del username, password, body

        response["content"] = json.loads(response["content"])

        # TODO : We could log the expiration Date
        # expiration_date: str = content["expirationDate"]

        # NOTE: Should we raise an exception if the content is not a valid
        #       JSON ?
        token: Optional[
            str] = response["content"]["token"] if response["status"] else None

        return token

    # TODO : Rename this to update_class_token() or update_auth_token()
    def update_token(self, token: Optional[str]) -> None:
        """

        Update the authorization token in the class headers and class
        attributes.

        Args :
            token (Optional[str]) : The authorization token to update in
                headers and class attributes.

                If the token is None, it will locally delete the last token
                from the class attributes.

        """
        self.auth_token = token

        if self.auth_token is not None:
            # If a new token is provided, we update the headers
            self.headers[self.api_token_name] = self.auth_token
        else:
            # If the token provided is None, we remove the last token from the
            # headers.
            _ = self.headers.pop(
                self.api_token_name
            ) if self.api_token_name in self.headers.keys() else None

        return

    def get_api_info(self) -> str:
        """

        Gets the current info about CLI API.

        NOTE : To get the info, the client should be already authorized.

        Returns:

            (str) :

        """
        try:
            response: Dict[str, Any] = get(
                url=self.info_url,
                headers=self.headers,
            )

        except Exception as e:
            # TODO: Verify if we need to raise an exception.
            response = {
                "url": self.info_url,
                "content": str(e),
                "status": False
            }

        return response["content"]

    def is_authorized(self):
        # TODO : Try with get_api_info()
        raise NotImplementedError

    def get_current_user(self):
        """

        Gets the current user based on the header token.

        Returns:
            ( ) :
        """
        # TODO : implement a method to get the expiration date of the current
        #        token
        response: Dict[str, Any] = get(url=self.auth_url, headers=self.headers)
        response["content"] = json.loads(response["content"])

        return response

    # Laboratories Endpoints

    def get_laboratories(self) -> List[Dict[str, Any]]:
        """

        Get all available laboratories for the current user.

        Returns :
            () : A list of laboratories objects.

        """
        response: Dict[str, Any] = get(url=self.labs_url, headers=self.headers)

        # response["content"] = [{"id" : str, "name": str}, ...]
        response["content"] = json.loads(response["content"])

        return response["content"]

    def select_laboratory(
        self,
        lab_name: Optional[str] = None,
        lab_id: Optional[int] = None,
    ) -> None:
        """ Sets the selected laboratory and adds it to the instance headers.

        Changes the header from internal class state with the id of the selected lab.
        This method will raise an error if the identifier (lab_name or lab_id) is not
        found.

        Args:
            lab_name (str): The name of the lab. If not set, the method will
                use the lab_id parameter. If both parameters are ommited, Lab
                is set to "Common".
            lab_id (int): ID of the lab. If not set the method will use the
                lab_name parameter as lab identifier
        """
        identifier = lab_name if lab_id is None else str(lab_id)
        search_field = 'name' if lab_id is None else 'id'
        if identifier is None:
            raise ValueError("Received None lab identifiers")
        if isinstance(identifier, str) and identifier.lower() == 'common':
            self.unselect_laboratory()
            return
        labs = self.get_laboratories()
        lab = list(filter(lambda x: x[search_field] == identifier, labs))
        if len(lab) == 0:
            raise IOError(
                f"Can't find {search_field} {identifier}. Available labs are {labs}"
            )
        # Finally store labid in headers
        self.headers.update({"tg-active-lab-id": str(lab[0]['id'])})
        print(f"Selected Lab: {lab[0]['name']}")

    def unselect_laboratory(self) -> None:
        """ Clear the selection of a laboratory and removes it from instance headers."""
        if "tg-active-lab-id" in self.headers:
            # Removing the lab header is interpreted as Common lab in the server
            del self.headers["tg-active-lab-id"]
        print(f"Selected Lab: Common")



#!/usr/bin/env python3
import json
from pathlib import Path
from typing import Any, Dict, List, Optional, Union
from urllib.parse import urlencode

import pytest
import requests_mock
from teselagen.api.client import get
from teselagen.api.client import post
from teselagen.api.client import TeselaGenClient
from teselagen.api.design_client import DESIGNClient
from teselagen.utils import load_from_json

# RBS MOCK DATA. These IDs are safe to be public.
JOB_ID_ONE = 'lowxt1rzramybxeelijsctypix9vk6fl'
JOB_ID_TWO = 'ouqzbuviolphyjasg0syhkseq6anltxz'

#@pytest.mark.incremental
class TestDESIGNClient():
    @pytest.fixture
    def client(self, host_url, api_token_name) -> TeselaGenClient:
        """

        A TeselaGen client instance.

        Returns:
            (TeselaGenClient) : An instance of TeselaGen client.

        """
        tg_client = TeselaGenClient(api_token_name=api_token_name, host_url=host_url)
        return tg_client

    @pytest.fixture
    def logged_client(self, client: TeselaGenClient) -> DESIGNClient:
        """

        A logged TEST client instance.

        Returns:
            (TESTClient) : An instance of the TEST client.

        """
        expiration_time: str = "30m"
        client.login(expiration_time=expiration_time)
        return client.design

    def test_get_assembly_report_mock(self, tmpdir, logged_client: DESIGNClient, requests_mock):
        """Checks report can be downloaded
        TODO: Requires a specific ID! A new endpoint for listing IDS should be implemented!
        """
        TEST_REPORT_ID=1023
        # Create Mock
        api_url_base = f"{logged_client.host_url}/design/cli-api"
        url = f"{api_url_base}{logged_client.URL_GET_ASSEMBLY_REPORT}/{TEST_REPORT_ID}"
        requests_mock.get(url, content=b"estoesunarchivobinario")
        # Create temporary folder
        local_filename = tmpdir.mkdir('assembly_report').join(f"report_{TEST_REPORT_ID}.zip")
        # Download report and make assertions
        report_filepath = logged_client.get_assembly_report(
            report_id=TEST_REPORT_ID,
            local_filename=local_filename
        )
        assert Path(report_filepath).is_file()

    @pytest.mark.skip("This test should be skipped until we have some way to ensure there is a report in database")
    def test_get_assembly_report(self, tmpdir, logged_client: DESIGNClient):
        """Checks report can be downloaded
        TODO: Requires a specific ID! A new endpoint for listing IDS should be implemented!
        """
        TEST_REPORT_ID=1023
        # Create temporary folder
        local_filename = tmpdir.mkdir('assembly_report').join(f"report_{TEST_REPORT_ID}.zip")
        # Download report and make assertions
        report_filepath = logged_client.get_assembly_report(
            report_id=TEST_REPORT_ID,
            local_filename=local_filename
        )
        assert Path(report_filepath).is_file()

    def test_get_dna_sequence_mock(self, logged_client: DESIGNClient, requests_mock):
        seq_id = 123807
        # Create Mock
        url = f"{logged_client.export_dna_sequence_url}json/{seq_id}"
        requests_mock.get(url, content=b'{"name": "pj5_00001"}')
        # Call method
        res = logged_client.get_dna_sequence(seq_id=seq_id)
        assert isinstance(res, dict)
        assert res['name'] == 'pj5_00001'

    @pytest.mark.skip("This test should be skipped until we know a dna sequence id in db")
    def test_get_dna_sequence(self, logged_client: DESIGNClient):
        seq_id = 123807
        res = logged_client.get_dna_sequence(seq_id=seq_id)
        assert isinstance(res, dict)
        assert res['name'] == 'pj5_00001'


    def test_get_dna_sequences(self, logged_client: DESIGNClient, requests_mock):
        expected_url = logged_client.export_dna_sequences_url + '?name=pj5_001'
        requests_mock.get(expected_url, content=b'[{"id": 12, "name": "hey", "sequence": "GATACA"}]')
        res = logged_client.get_dna_sequences(name='pj5_001')
        assert res == [{"id": 12, "name": "hey", "sequence": "GATACA"}]


    def test_get_designs(self, logged_client: DESIGNClient, requests_mock):
        # GET parameters
        params = {'gqlFilter': {
            'name': 'Gibson',
            'id':[12]}}
        # Build expected URL
        expected_params = params.copy()
        expected_params['gqlFilter'] = json.dumps(expected_params['gqlFilter'])
        expected_url = logged_client.get_designs_url + \
            "?" + urlencode(expected_params)
        # Prepare output from mock request
        requests_mock.get(expected_url, content=b'[{"id": 12, "name": "hola", "__typename": "design"}]')
        # Execute method
        res = logged_client.get_designs(
            name=params['gqlFilter']['name'],
            gql_filter={'id': params['gqlFilter']['id']})
        assert res == [{"id": 12, "name": "hola"}]


    ## RBS Calculator Tests
    def test_rbs_calculator_requires_token(self, logged_client: DESIGNClient):
        res = logged_client.rbs_calculator_status()
        assert type(res['error']) is Exception
        assert 'access is unauthorized' in str(res['error']) #TODO: Maybe there's a better way of checking for the specific unauthorized error.

    def test_rbs_calculator_jobs(self, logged_client: DESIGNClient):
        """ Hits a mock CLI API endpoint, it tests that its correctly calling it with the expected mock response. """
        api_url_base = f"{logged_client.host_url}/design/cli-api"
        mock_url = f"{api_url_base}/mock/rbs-calculator/jobs"
        res = get(url=mock_url, headers=logged_client.headers)
        res = json.loads(res["content"])
        assert sorted(list(res.keys())) == sorted(['authenticated', 'id_list', 'success'])
        assert res['authenticated'] == True
        assert res['success'] == True
        assert sorted(res['id_list']) == sorted([JOB_ID_ONE, JOB_ID_TWO])

    def test_rbs_calculator_organisms(self, logged_client: DESIGNClient):
        api_url_base = f"{logged_client.host_url}/design/cli-api"
        mock_url = f"{api_url_base}/mock/rbs-calculator/organisms"
        res = get(url=mock_url, headers=logged_client.headers)
        res = json.loads(res["content"])
        assert type(res) is list
        assert len(res) is 4
        assert sorted(list(res[0].keys())) == sorted(['accession', 'name'])

    def test_rbs_calculator_job(self, logged_client: DESIGNClient):
        api_url_base = f"{logged_client.host_url}/design/cli-api"
        mock_url = f"{api_url_base}/mock/rbs-calculator/jobs/{JOB_ID_ONE}"
        res = get(url=mock_url, headers=logged_client.headers)
        res = json.loads(res["content"])
        assert sorted(list(res.keys())) == sorted(['authenticated', 'inputData', 'jobInfo', 'message', 'outputData', 'success'])
        assert sorted(list(res['inputData'].keys())) == sorted(['algorithm', 'algorithm_version', 'long_UTR', 'mRNA', 'organism', 'title'])
        assert sorted(list(res['outputData'].keys())) == sorted(['Max_translation_initiation_rate', 'Min_translation_initiation_rate', 'Number_start_codons', 'ReverseRBS', 'TranslationRateAcrossPositions'])
        assert res['jobInfo']['jobId'] == JOB_ID_ONE

    def test_rbs_calculator_submit(self, logged_client: DESIGNClient):
        api_url_base = f"{logged_client.host_url}/design/cli-api"
        mock_url = f"{api_url_base}/mock/rbs-calculator/submit"
        params = json.dumps({"algorithm": "ReverseRBS"})
        res = post(url=mock_url, data=params, headers=logged_client.headers)
        res = json.loads(res["content"])
        assert sorted(list(res.keys())) == sorted(['authenticated', 'inputData', 'jobInfo', 'message', 'outputData', 'success'])
        assert sorted(list(res['inputData'].keys())) == sorted(['algorithm', 'algorithm_version', 'long_UTR', 'mRNA', 'organism', 'title'])
        assert res['outputData'] == {}
        assert res['jobInfo']['jobId'] == JOB_ID_ONE
